/**
Copyright (C) 2006  Volker Wiendl

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License version 2 as published 
by the Free Software Foundation.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program; 
if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
*/

#include "HtpcControl.h"
#include "PlugInManager.h"
#include "PlugInDialog.h"

#include "InputInterface.h"
#include <Qt/qdir.h>
#include <Qt/qtimer.h>
#include <Qt/qaction.h>
#include <Qt/qmenu.h>
#include <Qt/qmessagebox.h>
#include <Qt/qsystemtrayicon.h>
#ifdef WIN32
#include <windows.h>
#endif
HtpcControl::HtpcControl() : QObject(), m_plugin_manager(0)
{		
	m_sysTray = new QSystemTrayIcon(QIcon(QPixmap(":/Icons/Resources/HtpcControl.png")),this);
	m_sysTray->show();
	m_menuMain = new QMenu(tr("Main"));	
	m_menuMain->addAction(tr("Configure Plugins"), this, SLOT(showPlugIns()));
	m_menuMain->addAction(tr("About"), this, SLOT(about()));
	m_menuMain->addSeparator();
	m_menuMain->addAction(tr("Quit"), qApp, SLOT(quit()));
	init();
	m_sysTray->setContextMenu(m_menuMain);
}

HtpcControl::~HtpcControl(void)
{
	delete m_menuMain;
}

void HtpcControl::init()
{
	delete m_plugin_manager;
	m_plugin_manager = new PlugInManager(this);		
	if (!m_plugin_manager->init(QApplication::applicationDirPath()+"/config.xml"))
		QMessageBox::warning(0, tr("Fehler"), "Fehler beim Laden der Konfigurationsdatei:\n"+QApplication::applicationDirPath()+"/config.xml");
}

void HtpcControl::showPlugIns()
{
	PlugInDialog dlg;
	dlg.init(
		QApplication::applicationDirPath()+"/config.xml", 
		m_plugin_manager->getInputPlugIns(), 
		m_plugin_manager->getOutputPlugIns());
	if (dlg.exec()== QDialog::Accepted)
		m_plugin_manager->save(QApplication::applicationDirPath()+"/config.xml");
	else
		init();
}

void HtpcControl::about()
{
	QMessageBox::about(0, tr("About HtpcControl"), 
		tr("HtpcControl v.0.3\n\nCopyright (C) 2007 Volker Wiendl\n\nLicensed under the GPL v2"));
}
