/**
	Copyright (C) 2006  Volker Wiendl

    This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License version 2 as published 
	by the Free Software Foundation.
    This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
	See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along with this program; 
	if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
*/
#ifndef HTPCCONTROL_H_
#define HTPCCONTROL_H_

#include <Qt/qobject.h>
#include <Qt/qfileinfo.h>

class PlugInManager;
class QTimer;
class QSystemTrayIcon;
class QMenu;
/*! \mainpage HtpcControl 0.3
*
* \section intro_sec Introduction
*
* This application is the base application for several plugins used for controling a home theatre pc.
* \section concept_sec Concept
* There are two different plugin types:<br>
* InputInterface - Plugin: Creates "Input" for the Output Plugins. e.g. a music player creates data about the current playing song<br><br>
* OutputInterface - Plugin: Takes "input" to create a corresponding output action. e.g. an OSD takes the info about the current song and displays it<br>
* 
* @author wiendl, 05/02/2007
*/

/**
 * \brief Main Application
 *
 * The HtpcControl is the main application that will result in a system tray icon containing a context menu for
 * controling the different plugins and the application itself.
 */
class HtpcControl : public QObject
{
	Q_OBJECT
public:
	HtpcControl();
	virtual ~HtpcControl(void);

	void init();

private slots:
	void showPlugIns();
	void about();

private:
	QSystemTrayIcon*	m_sysTray;
	PlugInManager*		m_plugin_manager;	
	QTimer*				m_plugin_timer;

	QMenu*				m_menuMain;
	
};

#endif
