/**
	Copyright (C) 2006  Volker Wiendl

    This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License version 2 as published 
	by the Free Software Foundation.
    This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
	See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along with this program; 
	if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
*/
#ifndef OUTPUTINTERFACE_H_
#define OUTPUTINTERFACE_H_

#include <Qt/qstring.h>
#include <Qt/qplugin.h>
#include "../HTPCControl/HTPCCommand.h"
#include "../HTPCControl/HtpcInterface.h"

/**
 * \brief this class is for plugins that can handle different HtpcCommand instances send out by an InputInterface
 */
class OutputInterface : public HtpcInterface
{	
    public:
        virtual ~OutputInterface() {}
        
		/**
		 * has to be implemented in subclasses. It is supposed to return a List of all HtpcCommand supported
		 * by the output plugin.
		 */
		virtual const QList<HtpcCommand>& getCommands() const = 0;       
		/**
		 * This method will be called by InputInterfaces connected to this Outputplugin. It has to be implemented in subclasses.
		 * @return bool false in case of an error
		 */
		virtual bool receiveCommand(const HtpcCommand&) = 0;
};

Q_DECLARE_INTERFACE(OutputInterface,
                        "HTPCControl.OutputInterface/1.0")
#endif
