#include "ConfigDialog.h"

#include <qt/qmessagebox.h>
#include <qt/qdom.h>
#include <qt/qfile.h>
#include <qt/qtextstream.h>

ConfigDialog::ConfigDialog(QWidget* parent /*=0*/, Qt::WFlags wflags /*= 0*/) : QDialog(parent, wflags)
{
	setupUi(this);
	connect(m_helpMusicButton, SIGNAL(clicked()), this, SLOT(showMusicInfo()));
}

ConfigDialog::~ConfigDialog(void)
{
}

void ConfigDialog::loadConfig(const QString& configFile)
{
	QFile file(configFile);	
	QDomDocument doc("Settings");
	if (doc.setContent(&file))
	{
		// print out the element names of all elements that are direct children
		// of the outermost element.
		QDomElement docElem = doc.documentElement();
		QDomElement settings = docElem.namedItem("IowPlugin").toElement();
		/// enable RC5 special mode
		m_rc5->setChecked(settings.attribute("RC5","true").contains("true",Qt::CaseInsensitive));		
		/// switch on LED
		m_led->setChecked(settings.attribute("LED","true").contains("true",Qt::CaseInsensitive));
		/// enable LCD special mode
		m_lcd->setChecked(settings.attribute("LCD","true").contains("true",Qt::CaseInsensitive));	
		QDomElement lcd = settings.namedItem("LCD").toElement();
		m_lineWidth->setValue(lcd.attribute("LineWidth", "20").toInt());
		m_lineCount->setValue(lcd.attribute("LineCount", "4").toInt());
		m_displayUpdate->setValue(lcd.attribute("DisplayUpdate", "1.0").toFloat());
		m_alternateRows->setChecked(lcd.attribute("AlternateRowOrder","true").contains("true",Qt::CaseInsensitive));	
		m_musicInfoPattern->setText(lcd.attribute("MusicInfoPattern","%artist\\n%time\\nTitel: %title - Album: %album"));
		m_idleMessage->setText(lcd.attribute("IdleMessage"));
	}
	file.close();
}


void ConfigDialog::saveConfig(const QString &configFile)
{	
	QFile file(configFile);	
	if (file.open(QFile::WriteOnly | QFile::Truncate)) 
	{
        QTextStream out(&file);
		QDomDocument doc("Settings");		
		QDomElement rootNode = doc.createElement("Settings");
		doc.appendChild(rootNode);
		QDomElement settings = doc.createElement("IowPlugin");
		settings.setAttribute("RC5", m_rc5->checkState()==Qt::Checked ? "true" : "false");
		settings.setAttribute("LED", m_led->checkState()==Qt::Checked ? "true" : "false");
		settings.setAttribute("LCD", m_lcd->checkState()==Qt::Checked ? "true" : "false");
		QDomElement element = doc.createElement("LCD");
		element.setAttribute("LineWidth", QString::number(m_lineWidth->value()));
		element.setAttribute("LineCount", QString::number(m_lineCount->value()));
		element.setAttribute("DisplayUpdate", QString::number(m_displayUpdate->value()));
		element.setAttribute("AlternateRowOrder", m_alternateRows->checkState()==Qt::Checked ? "true" : "false");
		element.setAttribute("MusicInfoPattern", m_musicInfoPattern->text().replace("\n","\\n"));
		element.setAttribute("IdleMessage", m_idleMessage->text().replace("\n","\\n"));
		settings.appendChild(element);
		rootNode.appendChild(settings);
		doc.save(out, 6);
		file.close();

	}
	file.close();
}

void ConfigDialog::showMusicInfo()
{
	QMessageBox::information(this, tr("Help"), tr("Split several lines by \\n. If you want to\n use two following lines use % and the\n"
		"number of the line (e.g. %1 to use the first and second line as one)"));

}