#include "ConnectionDialog.h"

#include <qt/qmessagebox.h>

#include "../HTPCControl/OutputInterface.h"
#include "../HtpcControl/MusicInfo.h"
#include "../HtpcControl/SystemInfo.h"

ConnectionDialog::ConnectionDialog(QWidget* parent /*=0*/, Qt::WFlags wflags /*= 0*/) : QDialog(parent, wflags)
{
	setupUi(this);
	connect(m_outputCommands, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)), this, SLOT(itemSelected()));
	connect(m_winampCommands, SIGNAL(currentItemChanged(QTreeWidgetItem*, QTreeWidgetItem*)), this, SLOT(itemSelected()));
	connect(m_addCommandButton, SIGNAL(clicked()), this, SLOT(commandAssociated()));
	connect(m_removeCommandButton, SIGNAL(clicked()), this, SLOT(removeCommand()));
}

ConnectionDialog::~ConnectionDialog(void)
{
}

void ConnectionDialog::init(const QList<HtpcCommand>& musicListeners, const QList<HtpcCommand>& systemListeners, OutputInterface* outputPlugIn)
{
	m_musicListeners = musicListeners;
	m_systemListeners = systemListeners;
	QList<HtpcCommand>::const_iterator iter = musicListeners.constBegin();
	while(iter!=musicListeners.constEnd())
	{		
		if (iter->getPlugIn()->getPlugInName()==outputPlugIn->getPlugInName())
		{
			QTreeWidgetItem* musicItem = m_winampCommands->findItems("MusicInfo", Qt::MatchContains).at(0);
			QTreeWidgetItem* htpcItem = new QTreeWidgetItem(musicItem, QStringList(iter->getPlugIn()->getPlugInName()+" "+iter->getName()));		
			htpcItem->setData(0, Qt::UserRole, QVariant::fromValue(*iter));
		}
		++iter;
	}
	iter = systemListeners.constBegin();
	while(iter!=systemListeners.constEnd())
	{		
		if (iter->getPlugIn()->getPlugInName()==outputPlugIn->getPlugInName())
		{
			QTreeWidgetItem* systemItem = m_winampCommands->findItems("SystemInfo", Qt::MatchContains).at(0);
			QTreeWidgetItem* htpcItem = new QTreeWidgetItem(systemItem, QStringList(iter->getPlugIn()->getPlugInName()+" "+iter->getName()));		
			htpcItem->setData(0, Qt::UserRole, QVariant::fromValue(*iter));
		}
		++iter;
	}
	iter = outputPlugIn->getCommands().constBegin();
	while(iter!=outputPlugIn->getCommands().constEnd())
	{
		if (iter->getData().canConvert<MusicInfo>() || iter->getData().canConvert<SystemInfo>() || iter->getData().type() == QVariant::String)
		{
			QListWidgetItem* htpcItem = new QListWidgetItem(iter->getName(), m_outputCommands);
			htpcItem->setData(Qt::UserRole, QVariant::fromValue(*iter));
		}
		++iter;
	}
}

void ConnectionDialog::itemSelected()
{
	// check if we have a selection of two commands	
	if (m_winampCommands->currentItem() && m_outputCommands->currentItem())
	{		
		m_winampCommands->selectionModel()->select(m_winampCommands->currentIndex(), QItemSelectionModel::Select);
		m_outputCommands->selectionModel()->select(m_outputCommands->currentIndex(), QItemSelectionModel::Select);
		if (!m_outputCommands->currentItem()->data(Qt::UserRole).canConvert<HtpcCommand>())
		{
			m_addCommandButton->setEnabled(false);
			return;
		}
		const HtpcCommand& outputCommand = m_outputCommands->currentItem()->data(Qt::UserRole).value<HtpcCommand>();
		const QString& winampItem = m_winampCommands->currentItem()->text(0);
		// if we are connecting MusicInfo with MusicInfo, enable add button
		if (outputCommand.getData().canConvert<MusicInfo>() && winampItem=="MusicInfo")
			m_addCommandButton->setEnabled(true);
		// else if we are connecting SystemInfo with SystemInfo, enable add button
		else if (outputCommand.getData().canConvert<SystemInfo>() && winampItem=="SystemInfo")
			m_addCommandButton->setEnabled(true);
		// else if the output htpc command requests a string, enable add button
		else if ((winampItem=="MusicInfo" || winampItem=="SystemInfo") && outputCommand.getData().type() == QVariant::String)
			m_addCommandButton->setEnabled(true);
		// else no connection is supported
		else
			m_addCommandButton->setEnabled(false);
	}		
	else
		m_addCommandButton->setEnabled(false);
	if (m_winampCommands->currentItem())
	{
		const QString& winampItem = m_winampCommands->currentItem()->text(0);
		m_removeCommandButton->setEnabled(winampItem!="MusicInfo" && winampItem!="SystemInfo");
	}
	else
		m_removeCommandButton->setEnabled(false);
}

void ConnectionDialog::commandAssociated()
{
	QList<QListWidgetItem*>& htpcItems = m_outputCommands->selectedItems();
	QList<QTreeWidgetItem *>& winampItems = m_winampCommands->selectedItems();
	if (htpcItems.size() > 0 && winampItems.size() > 0)
	{
		HtpcCommand command = htpcItems.at(0)->data(Qt::UserRole).value<HtpcCommand>();
		QTreeWidgetItem* item = new QTreeWidgetItem(winampItems.at(0), QStringList(command.getPlugIn()->getPlugInName()+" "+command.getName()));
		item->setData(0, Qt::UserRole, QVariant::fromValue(command));		
		if (winampItems.at(0)->text(0)=="MusicInfo")
			m_musicListeners.push_back(command);
		else if (winampItems.at(0)->text(0)=="SystemInfo")
			m_systemListeners.push_back(command);
	}
}

void ConnectionDialog::removeCommand()
{
	QTreeWidgetItem* item = m_winampCommands->currentItem();
	if (item && item->parent())
	{
		QTreeWidgetItem* command = 0;
		if (item->parent()->text(0)=="MusicInfo")
		{                 
			command = item->parent()->takeChild(item->parent()->indexOfChild(item));
			int index = m_musicListeners.indexOf(command->data(0, Qt::UserRole).value<HtpcCommand>());
			if (index>=0) m_musicListeners.removeAt(index);			
		}
		else if (item->parent()->text(0)=="SystemInfo")
		{
			command = item->parent()->takeChild(item->parent()->indexOfChild(item));
			int index = m_systemListeners.indexOf(command->data(0, Qt::UserRole).value<HtpcCommand>());
			if (index>=0) m_systemListeners.removeAt(index);
		}
		delete command;
	}
}

void ConnectionDialog::getMusicListeners(QList<HtpcCommand>& listeners)
{
	listeners = m_musicListeners;	
}

void ConnectionDialog::getSystemListeners(QList<HtpcCommand> &listeners)
{
	listeners = m_systemListeners;
}

