/**
	Copyright (C) 2006  Volker Wiendl

    This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License version 2 as published 
	by the Free Software Foundation.
    This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
	See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along with this program; 
	if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
*/

#ifndef WINAMPPLUGIN_H_
#define WINAMPPLUGIN_H_

#include <qt/qobject.h>
#include <qt/qlist.h>
#include <qt/QMutex.h>
#include <qt/QReadWriteLock.h>
#include "../HTPCControl/HtpcCommand.h"
#include "../HTPCControl/InputInterface.h"
#include "../HTPCControl/OutputInterface.h"

#include <windows.h>

class WinampPlugIn : public InputInterface, public OutputInterface
{
	Q_OBJECT
	Q_INTERFACES(InputInterface OutputInterface)

public:
	WinampPlugIn();
	virtual ~WinampPlugIn();

	const QString getPlugInName() const;
	void configure();

	bool connectPlugIn(OutputInterface* output, const QDomElement& configuration);
	bool configPlugInConnection(OutputInterface* output, QDomElement& configuration);

	bool saveConfig(QDomDocument& doc);
	void stop();

	const QList<HtpcCommand>& getCommands() const {return m_commands;} 
	bool receiveCommand (const HtpcCommand&) ;
	

protected:

	void run();

private:
	void sendSystemInfo(const QString& message);

	void* allocateWinampBuffer(unsigned long bufsize);
	int   freeWinampBuffer(void *remoteBuf, unsigned long bufsize);
	unsigned long readWinampToLocal(void *remoteBuf, void *localBuf, unsigned long bufsize);
	unsigned long writeLocalToWinamp(void *localBuf, void *remoteBuf, unsigned long bufsize);
	int playingTrack();
	char* metaInfo(char *whichInfo, char *localBuffer);
	char* basicInfo(char* localBuf);

	/// commands the output interface understands
	QList<HtpcCommand> m_commands;
 
	/// input commands that were connected to the music info command
	QList<HtpcCommand>   m_musicListeners;	
	/// input commands that were connected to the system info command
	QList<HtpcCommand>   m_systemListeners;	
	
	/// duration for system info messages
	int					m_systemInfoDuration;

	volatile HWND m_hwndWinamp;
	volatile bool m_run;
	QReadWriteLock m_listenerLock;

	QMutex	 m_receiveLock;

};


#endif