#include "CommandDialog.h"

CommandDialog::CommandDialog(int commandOffset, QWidget* parent /*=0*/, Qt::WFlags flags /*=0*/) : QDialog(parent, flags), m_commandOffset(commandOffset)
{
	setupUi(this);
	connect(m_closeButton, SIGNAL(clicked()), this, SLOT(close()));
	connect(m_addButton, SIGNAL(clicked()), this, SLOT(addCommand()));
	connect(m_updateButton, SIGNAL(clicked()), this, SLOT(updateCommand()));
	connect(m_removeButton, SIGNAL(clicked()), this, SLOT(removeCommand()));
	connect(m_commands, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)), this, SLOT(itemChanged(QListWidgetItem*, QListWidgetItem*)));
}

CommandDialog::~CommandDialog()
{
}

void CommandDialog::addCommand(int id, const QString &command)
{	
	new QListWidgetItem(command, m_commands, id);
}

void CommandDialog::addCommand()
{
	new QListWidgetItem(m_program->text()+ " : " + m_params->text(), m_commands, m_commands->count() + m_commandOffset );
}

void CommandDialog::updateCommand()
{
	if (m_commands->currentItem())
	{
		QListWidgetItem* item = m_commands->currentItem();
		item->setText(m_program->text()+" : "+m_params->text());
		m_commands->setCurrentItem(0);
		m_commands->clearSelection();
	}
}

void CommandDialog::removeCommand()
{
	delete m_commands->takeItem(m_commands->currentRow());
}

void CommandDialog::close()
{
	setVisible(false);
}

void CommandDialog::itemChanged(QListWidgetItem* item, QListWidgetItem* prev)
{
	if (item)
	{
		m_program->setText(item->text().left(item->text().indexOf(" : ")));
		m_params->setText(item->text().right(item->text().length() - item->text().indexOf(" : ") - 3));		
	}
	m_updateButton->setEnabled(item!=0);
	m_removeButton->setEnabled(item!=0);
}

void CommandDialog::saveCommands(QDomElement &element)
{
	for (int i=0; i<m_commands->count(); ++i)
	{
		QListWidgetItem* item = m_commands->item(i);
		QDomElement command = element.toDocument().createElement("Command");
		command.setAttribute("id", item->type());
		command.setAttribute("value", item->text());
		element.appendChild(command);
	}
}