#include "OSD.h"

#include <qt/qApplication.h>
#include <qt/qtimer.h>
#include <qt/qdesktopWidget.h>
#include <qt/qpainter.h>
#include <WinUser.h>

#ifndef WS_EX_LAYERED
#define WS_EX_LAYERED           0x00080000
#define LWA_COLORKEY            0x00000001
#define LWA_ALPHA               0x00000002
#endif // ndef WS_EX_LAYERED


OSD::OSD(QWidget* parent /*= 0*/) : QWidget(parent, Qt::SplashScreen | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint ), m_useQueue(false),
m_font(QFont("Helvetica [Cronyx]",12))
{
    resize(640, 100);
	m_timer = new QTimer(this);
	m_timer->setSingleShot(true);
	connect(m_timer, SIGNAL(timeout()), this, SLOT(nextMessage()));
	QRect s = QApplication::desktop()->screenGeometry();
    move( s.width() / 2 - width()/2, s.height() - height()- 50);
	setAutoFillBackground(false);
	setAttribute(Qt::WA_NoSystemBackground);
	int exstyle = GetWindowLong(winId(), GWL_EXSTYLE);
	// Here we import the function from USER32.DLL
#if defined (_UNICODE) || defined (UNICODE)
	HMODULE hUser32 = GetModuleHandle(L"USER32.DLL");
#else
	HMODULE hUser32 = GetModuleHandle("USER32.DLL");
#endif
	m_pSetLayeredWindowAttributes = 
                       (lpfnSetLayeredWindowAttributes)GetProcAddress(hUser32, 
                       "SetLayeredWindowAttributes");
	// If the import did not succeed, make sure your app can handle it!
	if (NULL == m_pSetLayeredWindowAttributes)
	{
		exstyle |= WS_EX_TRANSPARENT;
		SetWindowLong(winId(), GWL_EXSTYLE, exstyle);
	}
	else
	{
		exstyle |= WS_EX_TRANSPARENT | WS_EX_LAYERED;
		SetWindowLong(winId(), GWL_EXSTYLE, exstyle);
		m_pSetLayeredWindowAttributes(winId(), RGB(255,255,255), 0, LWA_COLORKEY);
	}
	setColor("#00FF00");
}

OSD::~OSD(void)
{
}

void OSD::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);	
	painter.fillRect(rect(), QBrush(m_background));
	if (m_message.isEmpty())
		return;
	painter.setRenderHint(QPainter::TextAntialiasing);
	painter.setRenderHint(QPainter::Antialiasing);
    painter.setPen(m_color);	
	painter.setFont(m_font);	
	painter.drawText(rect(), Qt::AlignVCenter | Qt::AlignHCenter, m_message);
}

void OSD::nextMessage()
{	
	if (m_messages.size()==0 || m_durations.size()==0)
		hide();
	else
	{
		m_message = m_messages.takeFirst();
		m_timer->start(m_durations.takeFirst());
		update();
	}
}

void OSD::showMessage(const QString& message, int duration)
{
	if (!m_useQueue)
	{
		m_messages.clear();
		m_durations.clear();
	}
	m_messages.push_back(QString(message).remove(QRegExp("%\\d+\\s")));
	m_messages.push_back("");
	m_durations.push_back(duration);
	m_durations.push_back(5);
	if (!m_useQueue || !m_timer->isActive())
		nextMessage();
	setVisible(true);
}

void OSD::setColor(const QString &name)
{
	m_color = QColor(name);
	if (m_pSetLayeredWindowAttributes!=NULL)
	{
		m_background = QColor(m_color.red()==255 ? 254 : m_color.red()+1, m_color.green()==255 ? 254 : m_color.green()+1, m_color.blue()==255 ? 254 : m_color.blue()+1);
		m_pSetLayeredWindowAttributes(winId(), RGB(m_background.red(),m_background.green(),m_background.blue()), 0, LWA_COLORKEY);
	}
}