/**
	Copyright (C) 2006  Volker Wiendl

    This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License version 2 as published 
	by the Free Software Foundation.
    This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
	See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along with this program; 
	if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
*/
#ifndef WINDOWSPLUGIN_H_
#define WINDOWSPLUGIN_H_

#include <qt/qt_windows.h>
#include <qt/QReadWriteLock.h>
#include <qt/QMutex.h>
#include <qt/qmap.h>

#include "../HTPCControl/OutputInterface.h"
#include "../HTPCControl/InputInterface.h"

class OSD;
class QTimer;

class WindowsPlugIn : public InputInterface, public OutputInterface
{
	Q_OBJECT
	Q_INTERFACES(InputInterface OutputInterface)

public:
	WindowsPlugIn();
	virtual ~WindowsPlugIn();

	const QString getPlugInName() const;
	void configure();
	
	bool connectPlugIn(OutputInterface* output, const QDomElement& configuration);
	bool configPlugInConnection(OutputInterface* output, QDomElement& configuration);
	bool saveConfig(QDomDocument& doc);
	void stop();

	const QList<HtpcCommand>& getCommands() const {return m_commands;}
	bool receiveCommand (const HtpcCommand&);

public slots:
	void moveMouse(int dx, int dy);
	
signals:
	void message(const QString&, int);
	void mouseMoved(int, int);
	void shutDownStarted(int);
	void shutDownStoped();

protected:

	void run();

private slots:
	void deccelerateMouse();
	void shutDownTimeLeft();

private:

	void init();
	void sendSystemInfo(const QString& message, int duration);

	QList<HtpcCommand>	m_commands;
	QReadWriteLock		m_systemMessageLock;
	QList<HtpcCommand>  m_systemMessageListener;
	QString				m_lastDisplayText;
	QMutex				m_receiveLock;
	// On Screen Display
	OSD*				m_osd;	
	/// Timer for acceleration control
	QTimer*				m_timer;

	/// indicates if a shutdown has been started
	bool				m_shutdown;
	int					m_shutdownTimeLeft;
	QString				m_shutDownMessage;
	int					m_shutDownDelay;
	QTimer*				m_systemShutDownTimer;

	/// number of pixels the mouse will move in one step
	int					m_mouseAcceleration;
	/// Acceleration of mouse movements
	double				m_acceleration;

	QString				m_musicInfoPattern;
	int					m_musicInfoTimeout;
    const QString		m_pluginConfigFile;
	QMap<int,QString>	m_customCommands;

};


#endif